@@ -3,7 +3,7 @@ package main
 import (
 	"fmt"
 
-	"github.com/gosexy/gettext"
+	"github.com/chai2010/gettext-go/gettext"
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
