@@ -145,7 +145,7 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 		}
 		name = req.Name
 
-	} else {
+	} else if r.TLS != nil {
 
 		if len(r.TLS.PeerCertificates) < 1 {
 			return BadRequest(fmt.Errorf("No client certificate provided"))
@@ -158,6 +158,8 @@ func certificatesPost(d *Daemon, r *http.Request) Response {
 		}
 
 		name = remoteHost
+	} else {
+		return BadRequest(fmt.Errorf("Can't use TLS data on non-TLS link"))
 	}
 
 	fingerprint := shared.GenerateFingerprint(cert)
