@@ -4,6 +4,7 @@ import (
 	"bufio"
 	"fmt"
 	"os"
+	"os/exec"
 	"os/user"
 	"path"
 	"path/filepath"
@@ -19,9 +20,9 @@ import (
 type idmapEntry struct {
 	isuid    bool
 	isgid    bool
-	hostid   int
+	hostid   int // id as seen on the host - i.e. 100000
 	nsid     int // id as seen in the ns - i.e. 0
-	maprange int // id as seen on the host - i.e. 100000
+	maprange int
 }
 
 func (e *idmapEntry) ToLxcString() string {
@@ -99,6 +100,7 @@ func (e *idmapEntry) parse(s string) error {
 
 /*
  * Shift a uid from the host into the container
+ * I.e. 0 -> 1000 -> 101000
  */
 func (e *idmapEntry) shift_into_ns(id int) (int, error) {
 	if id < e.nsid || id >= e.nsid+e.maprange {
@@ -109,6 +111,19 @@ func (e *idmapEntry) shift_into_ns(id int) (int, error) {
 	return id - e.nsid + e.hostid, nil
 }
 
+/*
+ * Shift a uid from the container back to the host
+ * I.e. 101000 -> 1000
+ */
+func (e *idmapEntry) shift_from_ns(id int) (int, error) {
+	if id < e.hostid || id >= e.hostid+e.maprange {
+		// this mapping doesn't apply
+		return 0, fmt.Errorf("N/A")
+	}
+
+	return id - e.hostid + e.nsid, nil
+}
+
 /* taken from http://blog.golang.org/slices (which is under BSD licence) */
 func extend(slice []idmapEntry, element idmapEntry) []idmapEntry {
 	n := len(slice)
@@ -162,18 +177,30 @@ func (m IdmapSet) Append(s string) (IdmapSet, error) {
 	return m, nil
 }
 
-func (m IdmapSet) ShiftIntoNs(uid int, gid int) (int, int) {
+func (m IdmapSet) doShiftIntoNs(uid int, gid int, how string) (int, int) {
 	u := -1
 	g := -1
 	for _, e := range m.idmap {
+		var err error
+		var tmpu, tmpg int
 		if e.isuid && u == -1 {
-			tmpu, err := e.shift_into_ns(uid)
+			switch how {
+			case "in":
+				tmpu, err = e.shift_into_ns(uid)
+			case "out":
+				tmpu, err = e.shift_from_ns(uid)
+			}
 			if err == nil {
 				u = tmpu
 			}
 		}
 		if e.isgid && g == -1 {
-			tmpg, err := e.shift_into_ns(gid)
+			switch how {
+			case "in":
+				tmpg, err = e.shift_into_ns(gid)
+			case "out":
+				tmpg, err = e.shift_from_ns(gid)
+			}
 			if err == nil {
 				g = tmpg
 			}
@@ -183,6 +210,14 @@ func (m IdmapSet) ShiftIntoNs(uid int, gid int) (int, int) {
 	return u, g
 }
 
+func (m IdmapSet) ShiftIntoNs(uid int, gid int) (int, int) {
+	return m.doShiftIntoNs(uid, gid, "in")
+}
+
+func (m IdmapSet) ShiftFromNs(uid int, gid int) (int, int) {
+	return m.doShiftIntoNs(uid, gid, "out")
+}
+
 func getOwner(path string) (int, int, error) {
 	var stat syscall.Stat_t
 	err := syscall.Lstat(path, &stat)
@@ -194,13 +229,19 @@ func getOwner(path string) (int, int, error) {
 	return uid, gid, nil
 }
 
-func (set *IdmapSet) UidshiftIntoContainer(dir string, testmode bool) error {
+func (set *IdmapSet) doUidshiftIntoContainer(dir string, testmode bool, how string) error {
 	convert := func(path string, fi os.FileInfo, err error) (e error) {
 		uid, gid, err := getOwner(path)
 		if err != nil {
 			return err
 		}
-		newuid, newgid := set.ShiftIntoNs(uid, gid)
+		var newuid, newgid int
+		switch how {
+		case "in":
+			newuid, newgid = set.ShiftIntoNs(uid, gid)
+		case "out":
+			newuid, newgid = set.ShiftFromNs(uid, gid)
+		}
 		if testmode {
 			fmt.Printf("I would shift %q to %d %d\n", path, newuid, newgid)
 		} else {
@@ -224,8 +265,20 @@ func (set *IdmapSet) UidshiftIntoContainer(dir string, testmode bool) error {
 	return filepath.Walk(dir, convert)
 }
 
+func (set *IdmapSet) UidshiftIntoContainer(dir string, testmode bool) error {
+	return set.doUidshiftIntoContainer(dir, testmode, "in")
+}
+
+func (set *IdmapSet) UidshiftFromContainer(dir string, testmode bool) error {
+	return set.doUidshiftIntoContainer(dir, testmode, "out")
+}
+
 func (set *IdmapSet) ShiftRootfs(p string) error {
-	return set.UidshiftIntoContainer(p, false)
+	return set.doUidshiftIntoContainer(p, false, "in")
+}
+
+func (set *IdmapSet) UnshiftRootfs(p string) error {
+	return set.doUidshiftIntoContainer(p, false, "out")
 }
 
 const (
@@ -287,18 +340,30 @@ func DefaultIdmapSet() (*IdmapSet, error) {
 		return nil, err
 	}
 
-	umin, urange, err := getFromMap("/etc/subuid", me.Username)
-	if err != nil {
-		return nil, err
-	}
-	gmin, grange, err := getFromMap("/etc/subgid", me.Username)
-	if err != nil {
-		return nil, err
+	umin := 1000000
+	urange := 100000
+	gmin := 1000000
+	grange := 100000
+
+	newuidmap, _ := exec.LookPath("newuidmap")
+	newgidmap, _ := exec.LookPath("newgidmap")
+
+	if newuidmap != "" && newgidmap != "" {
+		umin, urange, err = getFromMap("/etc/subuid", me.Username)
+		if err != nil {
+			return nil, err
+		}
+
+		gmin, grange, err = getFromMap("/etc/subgid", me.Username)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	if urange < minIDRange {
 		return nil, fmt.Errorf("uidrange less than %d", minIDRange)
 	}
+
 	if grange < minIDRange {
 		return nil, fmt.Errorf("gidrange less than %d", minIDRange)
 	}
